<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 15/04/19
 * Time: 16.06
 */

namespace FiloBlu\Hotfix\Plugin\Model\Rule\Condition;

use Magento\Catalog\Model\ResourceModel\Product as ProductResource;
use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Catalog\Model\ProductFactory;
use Magento\Framework\App\ProductMetadataInterface;
use Magento\Framework\Model\AbstractModel;

/**
 * Class Product
 * @package FiloBlu\Hotfix\Plugin\Model\Rule\Condition
 */
class Product
{
    /**
     * @var string
     */
    protected $couponCode;
    /**
     * @var ProductResource
     */
    protected $productResource;
    /**
     * @var CategoryRepositoryInterface
     */
    protected $categoryRepository;
    /**
     * @var ProductFactory
     */
    protected $productFactory;
    /**
     * @var ProductMetadataInterface
     */
    protected $productMetadata;

    /**
     * Product constructor.
     * @param CategoryRepositoryInterface $categoryRepository
     * @param ProductResource $productResource
     * @param ProductFactory $productFactory
     * @param ProductMetadataInterface $productMetadata
     */
    public function __construct(CategoryRepositoryInterface $categoryRepository,
                                ProductResource $productResource,
                                ProductFactory $productFactory,
                                ProductMetadataInterface $productMetadata)
    {
        $this->categoryRepository = $categoryRepository;
        $this->productResource = $productResource;
        $this->productFactory = $productFactory;
        $this->productMetadata = $productMetadata;

    }

    /**
     * @param \Magento\SalesRule\Model\Rule\Condition\Product $product
     * @param $result
     * @param AbstractModel $model
     * @return bool
     */
    public function aroundValidate(\Magento\SalesRule\Model\Rule\Condition\Product $product, $result, AbstractModel $model)
    {

        if ($this->checkIfToApplyPlugin()) {

            //as written in \Magento\SalesRule\Model\Rule\Condition\Product :: validate
            ////@todo reimplement this method when is fixed MAGETWO-5713
            //MAGETWO-5713 merged into 2.3.1 . For more info see https://github.com/magento/magento2/commit/3de5d0e6e6df9e5d860fbdd1750347d7d606a204
            $myResult = false;
            if ($product->getAttribute() === 'category_ids') {

                $operator = $product->getOperator();
                $category = $product->getValueParsed();

                $this->couponCode = $model->getQuote()->getCouponCode();
                $productId = $model->getProductId();
                $myProduct = $this->productFactory->create()->setId($productId);
                $myProductCategory = $this->productResource->getCategoryIds($myProduct);

                //change operator for is one of and is not of one
                $operator = str_replace('()', '{}', $operator);

                switch ($operator) {
                    case '{}' :
                        foreach ($category as $value) {
                            if (in_array($value, $myProductCategory, false)) {
                                $myResult = true;
                            }
                        }
                        break;

                    case '!{}' :
                        foreach ($category as $value) {
                            if (!in_array($value, $myProductCategory, false)) {
                                $myResult = true;
                            }
                        }
                        break;

                    case '==' :
                        if ($category === $myProductCategory) {
                            $myResult = true;
                        }
                        break;

                    case '!=' :
                        if ($category !== $myProductCategory) {
                            $myResult = true;
                        }
                        break;
                }

                return $myResult;
            }

            return $result($model);
        }


    }

    /**
     * @return bool
     */
    public function checkIfToApplyPlugin()
    {
        if ($this->getVersion() < '2.3.1') {
            return true;
        }

        return false;
    }

    /**
     * @return string
     */
    public function getVersion()
    {
        return $this->productMetadata->getVersion();
    }

}